@@ -40,6 +40,7 @@
 #include <QCDEStyle>
 #include <QDialogButtonBox>
 #include <QCloseEvent>
+#include <QDesktopWidget>
 #ifdef Q_WS_WIN
   #include <QWindowsXPStyle>
 #endif
@@ -48,6 +49,9 @@
   #include <QMacStyle>
 #endif
 
+#include <time.h>
+#include <stdlib.h>
+
 #include "options_imp.h"
 #include "misc.h"
 
@@ -138,9 +142,11 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
   // General tab
   connect(checkNoSystray, SIGNAL(stateChanged(int)), this, SLOT(setSystrayOptionsState(int)));
   // Downloads tab
+  connect(checkTempFolder, SIGNAL(stateChanged(int)), this, SLOT(enableTempPathInput(int)));
   connect(checkScanDir, SIGNAL(stateChanged(int)), this, SLOT(enableDirScan(int)));
   connect(actionTorrentDlOnDblClBox, SIGNAL(currentIndexChanged(int)), this, SLOT(enableApplyButton()));
   connect(actionTorrentFnOnDblClBox, SIGNAL(currentIndexChanged(int)), this, SLOT(enableApplyButton()));
+  connect(checkTempFolder, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
   // Connection tab
   connect(checkUploadLimit, SIGNAL(stateChanged(int)), this, SLOT(enableUploadLimit(int)));
   connect(checkDownloadLimit,  SIGNAL(stateChanged(int)), this, SLOT(enableDownloadLimit(int)));
@@ -150,6 +156,7 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
   connect(checkMaxUploadsPerTorrent,  SIGNAL(stateChanged(int)), this, SLOT(enableMaxUploadsLimitPerTorrent(int)));
   connect(checkRatioLimit,  SIGNAL(stateChanged(int)), this, SLOT(enableShareRatio(int)));
   connect(checkRatioRemove,  SIGNAL(stateChanged(int)), this, SLOT(enableDeleteRatio(int)));
+  connect(checkSameDHTPort, SIGNAL(stateChanged(int)), this, SLOT(enableDHTPortSettings(int)));
   // Proxy tab
   connect(comboProxyType_http, SIGNAL(currentIndexChanged(int)),this, SLOT(enableProxyHTTP(int)));
   connect(checkProxyAuth_http,  SIGNAL(stateChanged(int)), this, SLOT(enableProxyAuthHTTP(int)));
@@ -183,8 +190,7 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
   connect(checkScanDir, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
   connect(textScanDir, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
   // Connection tab
-  connect(spinPortMin, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
-  connect(spinPortMax, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
+  connect(spinPort, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
   connect(checkUPnP, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
   connect(checkNATPMP, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
   connect(checkUploadLimit, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
@@ -199,6 +205,8 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
   connect(spinMaxConnecPerTorrent, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
   connect(spinMaxUploadsPerTorrent, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
   connect(checkDHT, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
+  connect(checkSameDHTPort, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
+  connect(spinDHTPort, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
   connect(checkLSD, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
   connect(checkAzureusSpoof, SIGNAL(stateChanged(int)), this, SLOT(enableApplyButton()));
   connect(comboEncryption, SIGNAL(currentIndexChanged(int)), this, SLOT(enableApplyButton()));
@@ -245,6 +253,10 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
     checkSystrayBalloons->setChecked(false);
     checkSystrayBalloons->setEnabled(false);
   }
+  // Tab selection mecanism
+  connect(tabSelection, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)));
+  // Adapt size
+  adaptToScreenSize();
 }
 
 // Main destructor
@@ -252,6 +264,12 @@ options_imp::~options_imp(){
   qDebug("-> destructing Options");
 }
 
+void options_imp::changePage(QListWidgetItem *current, QListWidgetItem *previous) {
+     if (!current)
+         current = previous;
+     tabOption->setCurrentIndex(tabSelection->row(current));
+ }
+
 void options_imp::useStyle(){
   int style = getStyle();
   switch(style) {
@@ -280,13 +298,29 @@ void options_imp::useStyle(){
   }
 }
 
+void options_imp::adaptToScreenSize() {
+  int scrn = 0;
+  QWidget *w = this->topLevelWidget();
+
+  if(w)
+    scrn = QApplication::desktop()->screenNumber(w);
+  else if(QApplication::desktop()->isVirtualDesktop())
+    scrn = QApplication::desktop()->screenNumber(QCursor::pos());
+  else
+    scrn = QApplication::desktop()->screenNumber(this);
+
+  QRect desk(QApplication::desktop()->availableGeometry(scrn));
+  if(width() > desk.width() || height() > desk.height()) {
+    if(desk.width() > 0 && desk.height() > 0)
+      resize(desk.width(), desk.height());
+  }
+}
+
 void options_imp::saveOptions(){
   applyButton->setEnabled(false);
   QSettings settings("qBittorrent", "qBittorrent");
   // Apply style
   useStyle();
-  // Check if min port < max port
-  checkPortsLogic();
   settings.beginGroup("Preferences");
   // General preferences
   settings.beginGroup("General");
@@ -306,6 +340,8 @@ void options_imp::saveOptions(){
   // Downloads preferences
   settings.beginGroup("Downloads");
   settings.setValue(QString::fromUtf8("SavePath"), getSavePath());
+  settings.setValue(QString::fromUtf8("TempPathEnabled"), isTempPathEnabled());
+  settings.setValue(QString::fromUtf8("TempPath"), getTempPath());
   settings.setValue(QString::fromUtf8("PreAllocation"), preAllocateAllFiles());
   settings.setValue(QString::fromUtf8("AdditionDialog"), useAdditionDialog());
   settings.setValue(QString::fromUtf8("StartInPause"), addTorrentsInPause());
@@ -316,8 +352,7 @@ void options_imp::saveOptions(){
   settings.endGroup();
   // Connection preferences
   settings.beginGroup("Connection");
-  settings.setValue(QString::fromUtf8("PortRangeMin"), getPorts().first);
-  settings.setValue(QString::fromUtf8("PortRangeMax"), getPorts().second);
+  settings.setValue(QString::fromUtf8("PortRangeMin"), getPort());
   settings.setValue(QString::fromUtf8("UPnP"), isUPnPEnabled());
   settings.setValue(QString::fromUtf8("NAT-PMP"), isNATPMPEnabled());
   settings.setValue(QString::fromUtf8("GlobalDLLimit"), getGlobalBandwidthLimits().first);
@@ -363,6 +398,8 @@ void options_imp::saveOptions(){
   settings.setValue(QString::fromUtf8("MaxConnecsPerTorrent"), getMaxConnecsPerTorrent());
   settings.setValue(QString::fromUtf8("MaxUploadsPerTorrent"), getMaxUploadsPerTorrent());
   settings.setValue(QString::fromUtf8("DHT"), isDHTEnabled());
+  settings.setValue(QString::fromUtf8("sameDHTPortAsBT"), isDHTPortSameAsBT());
+  settings.setValue(QString::fromUtf8("DHTPort"), getDHTPort());
   settings.setValue(QString::fromUtf8("LSD"), isLSDEnabled());
   settings.setValue(QString::fromUtf8("AzureusSpoof"), shouldSpoofAzureus());
   settings.setValue(QString::fromUtf8("Encryption"), getEncryptionSetting());
@@ -510,6 +547,15 @@ void options_imp::loadOptions(){
     home += QDir::separator();
   }
   textSavePath->setText(settings.value(QString::fromUtf8("SavePath"), home+"qBT_dir").toString());
+  if(settings.value(QString::fromUtf8("TempPathEnabled"), false).toBool()) {
+    // enable
+    checkTempFolder->setChecked(true);
+    enableTempPathInput(2);
+  } else {
+    checkTempFolder->setChecked(false);
+    enableTempPathInput(0);
+  }
+  textTempPath->setText(settings.value(QString::fromUtf8("TempPath"), home+"qBT_dir"+QDir::separator()+"temp").toString());
   checkPreallocateAll->setChecked(settings.value(QString::fromUtf8("PreAllocation"), false).toBool());
   checkAdditionDialog->setChecked(settings.value(QString::fromUtf8("AdditionDialog"), true).toBool());
   checkStartPaused->setChecked(settings.value(QString::fromUtf8("StartInPause"), false).toBool());
@@ -531,10 +577,7 @@ void options_imp::loadOptions(){
   settings.endGroup();
   // Connection preferences
   settings.beginGroup("Connection");
-  spinPortMin->setValue(settings.value(QString::fromUtf8("PortRangeMin"), 6881).toInt());
-  spinPortMax->setValue(settings.value(QString::fromUtf8("PortRangeMax"), 6889).toInt());
-  // Check if min port < max port
-  checkPortsLogic();
+  spinPort->setValue(settings.value(QString::fromUtf8("PortRangeMin"), 6881).toInt());
   checkUPnP->setChecked(settings.value(QString::fromUtf8("UPnP"), true).toBool());
   checkNATPMP->setChecked(settings.value(QString::fromUtf8("NAT-PMP"), true).toBool());
   intValue = settings.value(QString::fromUtf8("GlobalDLLimit"), -1).toInt();
@@ -654,6 +697,9 @@ void options_imp::loadOptions(){
     spinMaxUploadsPerTorrent->setEnabled(false);
   }
   checkDHT->setChecked(settings.value(QString::fromUtf8("DHT"), true).toBool());
+  checkSameDHTPort->setChecked(settings.value(QString::fromUtf8("sameDHTPortAsBT"), true).toBool());
+  enableDHTPortSettings(checkSameDHTPort->checkState());
+  spinDHTPort->setValue(settings.value(QString::fromUtf8("DHTPort"), 6882).toInt());
   checkLSD->setChecked(settings.value(QString::fromUtf8("LSD"), true).toBool());
   checkAzureusSpoof->setChecked(settings.value(QString::fromUtf8("AzureusSpoof"), false).toBool());
   comboEncryption->setCurrentIndex(settings.value(QString::fromUtf8("Encryption"), 0).toInt());
@@ -727,12 +773,19 @@ void options_imp::loadOptions(){
   textWebUiPassword->setText(settings.value("Password", "").toString());
   // End Web UI
   settings.endGroup();
+  // Random stuff
+  srand(time(0));
 }
 
 // return min & max ports
 // [min, max]
-std::pair<unsigned short, unsigned short> options_imp::getPorts() const{
-  return std::make_pair(spinPortMin->value(), spinPortMax->value());
+int options_imp::getPort() const{
+  return spinPort->value();
+}
+
+void options_imp::on_randomButton_clicked() {
+  // Range [1024: 65535]
+  spinPort->setValue(rand() % 64512 + 1024);
 }
 
 int options_imp::getEncryptionSetting() const{
@@ -825,6 +878,12 @@ bool options_imp::systrayIntegration() const{
   return (!checkNoSystray->isChecked());
 }
 
+int options_imp::getDHTPort() const {
+if(isDHTPortSameAsBT())
+  return 0;
+return spinDHTPort->value();
+}
+
 // Return Share ratio
 float options_imp::getDesiredRatio() const{
   if(checkRatioLimit->isChecked()){
@@ -857,6 +916,14 @@ QString options_imp::getSavePath() const{
   return textSavePath->text();
 }
 
+QString options_imp::getTempPath() const {
+    return textTempPath->text();
+}
+
+bool options_imp::isTempPathEnabled() const {
+    return checkTempFolder->isChecked();
+}
+
 // Return max connections number
 int options_imp::getMaxConnecs() const{
   if(!checkMaxConnecs->isChecked()){
@@ -913,7 +980,7 @@ void options_imp::on_buttonBox_rejected(){
 }
 
 void options_imp::enableDownloadLimit(int checkBoxValue){
-  if(checkBoxValue!=2){
+  if(checkBoxValue != 2){
     //Disable
     spinDownloadLimit->setEnabled(false);
   }else{
@@ -922,6 +989,18 @@ void options_imp::enableDownloadLimit(int checkBoxValue){
   }
 }
 
+void options_imp::enableTempPathInput(int checkBoxValue){
+  if(checkBoxValue != 2){
+    //Disable
+    textTempPath->setEnabled(false);
+    browseTempDirButton->setEnabled(false);
+  }else{
+    //enable
+    textTempPath->setEnabled(true);
+    browseTempDirButton->setEnabled(true);
+  }
+}
+
 bool options_imp::useAdditionDialog() const{
   return checkAdditionDialog->isChecked();
 }
@@ -1046,6 +1125,18 @@ void options_imp::enableShareRatio(int checkBoxValue){
   }
 }
 
+void options_imp::enableDHTPortSettings(int checkBoxValue) {
+if(checkBoxValue == 2){
+    //Disable
+    spinDHTPort->setEnabled(false);
+    dh_port_lbl->setEnabled(false);
+  }else{
+    //enable
+    spinDHTPort->setEnabled(true);
+    dh_port_lbl->setEnabled(true);
+  }
+}
+
 void options_imp::enableDeleteRatio(int checkBoxValue){
   if(checkBoxValue != 2){
     //Disable
@@ -1152,6 +1243,10 @@ bool options_imp::addTorrentsInPause() const {
   return checkStartPaused->isChecked();
 }
 
+bool options_imp::isDHTPortSameAsBT() const {
+  return checkSameDHTPort->isChecked();
+}
+
 // Proxy settings
 bool options_imp::isProxyEnabled() const{
   return comboProxyType->currentIndex();
@@ -1225,15 +1320,6 @@ void options_imp::setLocale(QString locale){
   }
 }
 
-// Is called before saving to check if minPort < maxPort
-void options_imp::checkPortsLogic(){
-  int maxValue = spinPortMax->value();
-  if(spinPortMin->value() > spinPortMax->value()){
-    spinPortMax->setValue(spinPortMin->value());
-    spinPortMin->setValue(maxValue);
-  }
-}
-
 // Return scan dir set in options
 QString options_imp::getScanDir() const {
   if(checkScanDir->isChecked()){
