@@ -39,6 +39,9 @@
 #include <iostream>
 #include <QTimer>
 #include <QDir>
+#include <QMenu>
+#include <QClipboard>
+#include <QMimeData>
 
 #include "searchEngine.h"
 #include "bittorrent.h"
@@ -53,12 +56,11 @@ SearchEngine::SearchEngine(bittorrent *BTSession, QSystemTrayIcon *myTrayIcon, b
   setupUi(this);
   // new qCompleter to the search pattern
   startSearchHistory();
-  searchCompleter = new QCompleter(searchHistory, this);
-  searchCompleter->setCaseSensitivity(Qt::CaseInsensitive);
-  search_pattern->setCompleter(searchCompleter);
+  searchCompleter = 0;
+  createCompleter();
   // Add close tab button
   closeTab_button = new QPushButton();
-  closeTab_button->setIcon(QIcon(QString::fromUtf8(":/Icons/gnome-shutdown.png")));
+  closeTab_button->setIcon(QIcon(QString::fromUtf8(":/Icons/oxygen/tab-close.png")));
   closeTab_button->setFlat(true);
   connect(closeTab_button, SIGNAL(clicked()), this, SLOT(closeTab_button_clicked()));
   tabWidget->setCornerWidget(closeTab_button);
@@ -74,11 +76,27 @@ SearchEngine::SearchEngine(bittorrent *BTSession, QSystemTrayIcon *myTrayIcon, b
   connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(tab_changed(int)));
   searchTimeout = new QTimer(this);
   searchTimeout->setSingleShot(true);
-  connect(searchTimeout, SIGNAL(timeout()), this, SLOT(on_stop_search_button_clicked()));
-  // Check last enabled search engines
-  loadEngineSettings();
+  connect(searchTimeout, SIGNAL(timeout()), this, SLOT(on_search_button_clicked()));
   // Update nova.py search plugin if necessary
   updateNova();
+  supported_engines = new SupportedEngines();
+  // Fill in category combobox
+  fillCatCombobox();
+  connect(search_pattern, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(displayPatternContextMenu(QPoint)));
+}
+
+void SearchEngine::fillCatCombobox() {
+  comboCategory->clear();
+  comboCategory->addItem(full_cat_names["all"], QVariant("all"));
+  QStringList supported_cat = supported_engines->supportedCategories();
+  foreach(QString cat, supported_cat) {
+    qDebug("Supported category: %s", cat.toLocal8Bit().data());
+    comboCategory->addItem(full_cat_names[cat], QVariant(cat));
+  }
+}
+
+QString SearchEngine::selectedCategory() const {
+  return comboCategory->itemData(comboCategory->currentIndex()).toString();
 }
 
 SearchEngine::~SearchEngine(){
@@ -87,32 +105,76 @@ SearchEngine::~SearchEngine(){
   saveSearchHistory();
   searchProcess->kill();
   searchProcess->waitForFinished();
-	foreach(QProcess *downloader, downloaders) {
-		downloader->kill();
-		downloader->waitForFinished();
-		delete downloader;
-	}
+  foreach(QProcess *downloader, downloaders) {
+    downloader->kill();
+    downloader->waitForFinished();
+    delete downloader;
+  }
   delete searchTimeout;
   delete searchProcess;
-  delete searchCompleter;
+  delete supported_engines;
+  if(searchCompleter)
+    delete searchCompleter;
+}
+
+void SearchEngine::displayPatternContextMenu(QPoint) {
+  QMenu myMenu(this);
+  QAction cutAct(QIcon(":/Icons/oxygen/edit-cut.png"), tr("Cut"), &myMenu);
+  QAction copyAct(QIcon(":/Icons/oxygen/edit-copy.png"), tr("Copy"), &myMenu);
+  QAction pasteAct(QIcon(":/Icons/oxygen/edit-paste.png"), tr("Paste"), &myMenu);
+  QAction clearAct(QIcon(":/Icons/oxygen/edit_clear.png"), tr("Clear field"), &myMenu);
+  QAction clearHistoryAct(QIcon(":/Icons/oxygen/edit-clear.png"), tr("Clear completion history"), &myMenu);
+  bool hasCopyAct = false;
+  if(search_pattern->hasSelectedText()) {
+    myMenu.addAction(&cutAct);
+    myMenu.addAction(&copyAct);
+    hasCopyAct = true;
+  }
+  if(qApp->clipboard()->mimeData()->hasText()) {
+    myMenu.addAction(&pasteAct);
+    hasCopyAct = true;
+  }
+  if(hasCopyAct)
+    myMenu.addSeparator();
+  myMenu.addAction(&clearHistoryAct);
+  myMenu.addAction(&clearAct);
+  QAction *act = myMenu.exec(QCursor::pos());
+  if(act != 0) {
+    if(act == &clearHistoryAct) {
+      searchHistory.clear();
+      createCompleter();
+    } else if (act == &pasteAct) {
+    } else if (act == &pasteAct) {
+      search_pattern->paste();
+    }
+    else if (act == &cutAct) {
+      search_pattern->cut();
+    }
+    else if (act == &copyAct) {
+      search_pattern->copy();
+    }
+    else if (act == &clearAct) {
+      search_pattern->clear();
+    }
+  }
 }
 
 void SearchEngine::tab_changed(int t)
 {//when we switch from a tab that is not empty to another that is empty the download button 
-	//doesn't have to be available
-	if(t>-1)
-	{//-1 = no more tab
+  //doesn't have to be available
+  if(t>-1)
+  {//-1 = no more tab
     if(all_tab.at(tabWidget->currentIndex())->getCurrentSearchListModel()->rowCount()) {
-        download_button->setEnabled(true);
+      download_button->setEnabled(true);
     } else {
-        download_button->setEnabled(false);
+      download_button->setEnabled(false);
     }
-	}
+  }
 }
 
 void SearchEngine::on_enginesButton_clicked() {
-  engineSelectDlg *dlg = new engineSelectDlg(this);
-  connect(dlg, SIGNAL(enginesChanged()), this, SLOT(loadEngineSettings()));
+  engineSelectDlg *dlg = new engineSelectDlg(this, supported_engines);
+  connect(dlg, SIGNAL(enginesChanged()), this, SLOT(fillCatCombobox()));
 }
 
 // get the last searchs from a QSettings to a QStringList
@@ -123,23 +185,6 @@ void SearchEngine::startSearchHistory(){
   settings.endGroup();
 }
 
-void SearchEngine::loadEngineSettings() {
-  qDebug("Loading engine settings");
-  enabled_engines.clear();
-  QSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
-  QStringList known_engines = settings.value(QString::fromUtf8("SearchEngines/knownEngines"), QStringList()).toStringList();
-  QVariantList known_enginesEnabled = settings.value(QString::fromUtf8("SearchEngines/knownEnginesEnabled"), QList<QVariant>()).toList();
-  unsigned int i = 0;
-  foreach(const QString &engine, known_engines) {
-    if(known_enginesEnabled.at(i).toBool())
-      enabled_engines << engine;
-    ++i;
-  }
-  if(enabled_engines.empty())
-    enabled_engines << "all";
-  qDebug("Engine settings loaded");
-}
-
 // Save the history list into the QSettings for the next session
 void SearchEngine::saveSearchHistory()
 {
@@ -152,11 +197,13 @@ void SearchEngine::saveSearchHistory()
 // Function called when we click on search button
 void SearchEngine::on_search_button_clicked(){
   if(searchProcess->state() != QProcess::NotRunning){
-    searchProcess->kill();
-    searchProcess->waitForFinished();
-  }
-  if(searchTimeout->isActive()) {
-    searchTimeout->stop();
+    searchProcess->terminate();
+    search_stopped = true;
+    if(searchTimeout->isActive()) {
+      searchTimeout->stop();
+    }
+    search_button->setText("Search");
+    return;
   }
   QString pattern = search_pattern->text().trimmed();
   // No search pattern entered
@@ -166,6 +213,7 @@ void SearchEngine::on_search_button_clicked(){
   }
   // Tab Addition
   currentSearchTab=new SearchTab(this);
+  connect(currentSearchTab->header(), SIGNAL(sectionResized(int, int, int)), this, SLOT(propagateSectionResized(int,int,int)));
   all_tab.append(currentSearchTab);
   tabWidget->addTab(currentSearchTab, pattern);
   tabWidget->setCurrentWidget(currentSearchTab);
@@ -177,19 +225,17 @@ void SearchEngine::on_search_button_clicked(){
     // verify the max size of the history
     if(searchHistory.size() > SEARCHHISTORY_MAXSIZE)
       searchHistory = searchHistory.mid(searchHistory.size()/2,searchHistory.size()/2);
-    searchCompleter = new QCompleter(searchHistory, this);
-    searchCompleter->setCaseSensitivity(Qt::CaseInsensitive);
-    search_pattern->setCompleter(searchCompleter);
+    createCompleter();
   }
 
-
   // Getting checked search engines
-  Q_ASSERT(!enabled_engines.empty());
   QStringList params;
   QStringList engineNames;
   search_stopped = false;
   params << misc::qBittorrentPath()+"search_engine"+QDir::separator()+"nova2.py";
-  params << enabled_engines.join(",");
+  params << supported_engines->enginesEnabled().join(",");
+  qDebug("Search with category: %s", selectedCategory().toLocal8Bit().data());
+  params << selectedCategory();
   params << pattern.split(" ");
   // Update SearchEngine widgets
   no_search_results = true;
@@ -202,41 +248,67 @@ void SearchEngine::on_search_button_clicked(){
   searchTimeout->start(180000); // 3min
 }
 
+void SearchEngine::createCompleter() {
+  if(searchCompleter)
+    delete searchCompleter;
+  searchCompleter = new QCompleter(searchHistory, this);
+  searchCompleter->setCaseSensitivity(Qt::CaseInsensitive);
+  search_pattern->setCompleter(searchCompleter);
+}
+
+void SearchEngine::propagateSectionResized(int index, int , int newsize) {
+  foreach(SearchTab * tab, all_tab) {
+    tab->getCurrentTreeView()->setColumnWidth(index, newsize);
+  }
+  saveResultsColumnsWidth();
+}
+
+void SearchEngine::saveResultsColumnsWidth() {
+  if(all_tab.size() > 0) {
+    QTreeView* treeview = all_tab.first()->getCurrentTreeView();
+    QSettings settings("qBittorrent", "qBittorrent");
+    QStringList width_list;
+    QStringList new_width_list;
+    short nbColumns = all_tab.first()->getCurrentSearchListModel()->columnCount();
+
+    QString line = settings.value("SearchResultsColsWidth", QString()).toString();
+    if(!line.isEmpty()) {
+      width_list = line.split(' ');
+    }
+    for(short i=0; i<nbColumns; ++i){
+      if(treeview->columnWidth(i)<1 && width_list.size() == nbColumns && width_list.at(i).toInt()>=1) {
+        // load the former width
+        new_width_list << width_list.at(i);
+      } else if(treeview->columnWidth(i)>=1) {
+        // usual case, save the current width
+        new_width_list << QString::fromUtf8(misc::toString(treeview->columnWidth(i)).c_str());
+      } else {
+        // default width
+        treeview->resizeColumnToContents(i);
+        new_width_list << QString::fromUtf8(misc::toString(treeview->columnWidth(i)).c_str());
+      }
+    }
+    settings.setValue("SearchResultsColsWidth", new_width_list.join(" "));
+  }
+}
+
 void SearchEngine::downloadTorrent(QString engine_url, QString torrent_url) {
-		QProcess *downloadProcess = new QProcess(this);
-		connect(downloadProcess, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(downloadFinished(int,QProcess::ExitStatus)));
-		downloaders << downloadProcess;
-		QStringList params;
-		params << misc::qBittorrentPath()+"search_engine"+QDir::separator()+"nova2dl.py";
-		params << engine_url;
-		params << torrent_url;
-		// Launch search
-		downloadProcess->start("python", params, QIODevice::ReadOnly);
+  QProcess *downloadProcess = new QProcess(this);
+  connect(downloadProcess, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(downloadFinished(int,QProcess::ExitStatus)));
+  downloaders << downloadProcess;
+  QStringList params;
+  params << misc::qBittorrentPath()+"search_engine"+QDir::separator()+"nova2dl.py";
+  params << engine_url;
+  params << torrent_url;
+  // Launch search
+  downloadProcess->start("python", params, QIODevice::ReadOnly);
 }
 
 void SearchEngine::searchStarted(){
   // Update SearchEngine widgets
   search_status->setText(tr("Searching..."));
   search_status->repaint();
-  stop_search_button->setEnabled(true);
-  stop_search_button->repaint();
-  // clear results window ... not needed since we got Tabbed
-  //SearchListModel->removeRows(0, SearchListModel->rowCount());
-  // Clear previous results urls too
-  //searchResultsUrls.clear();
-}
-
-// Download the given item from search results list
-void SearchEngine::downloadSelectedItem(const QModelIndex& index){
-  int row = index.row();
-  // Get Item url
-  QStandardItemModel *model = all_tab.at(tabWidget->currentIndex())->getCurrentSearchListModel();
-	QString engine_url = model->data(model->index(index.row(), ENGINE_URL_COLUMN)).toString();
-  QString torrent_url = model->data(model->index(index.row(), URL_COLUMN)).toString();
-  // Download from url
-  downloadTorrent(engine_url, torrent_url);
-  // Set item color to RED
-  all_tab.at(tabWidget->currentIndex())->setRowColor(row, "red");
+  search_button->setText("Stop");
 }
 
 // search Qprocess return output as soon as it gets new
@@ -258,19 +330,19 @@ void SearchEngine::readSearchOutput(){
 }
 
 void SearchEngine::downloadFinished(int exitcode, QProcess::ExitStatus) {
-		QProcess *downloadProcess = (QProcess*)sender();
-		if(exitcode == 0) {
-				QString line = QString::fromUtf8(downloadProcess->readAllStandardOutput()).trimmed();
-				QStringList parts = line.split(' ');
-				if(parts.size() == 2) {
-						QString path = parts[0];
-						QString url = parts[1];
-						BTSession->processDownloadedFile(url, path);
-				}
-		}
-		qDebug("Deleting downloadProcess");
-		downloaders.removeOne(downloadProcess);
-		delete downloadProcess;
+  QProcess *downloadProcess = (QProcess*)sender();
+  if(exitcode == 0) {
+    QString line = QString::fromUtf8(downloadProcess->readAllStandardOutput()).trimmed();
+    QStringList parts = line.split(' ');
+    if(parts.size() == 2) {
+      QString path = parts[0];
+      QString url = parts[1];
+      BTSession->processDownloadedFile(url, path);
+    }
+  }
+  qDebug("Deleting downloadProcess");
+  downloaders.removeAll(downloadProcess);
+  delete downloadProcess;
 }
 
 // Update nova.py search plugin if necessary
@@ -300,8 +372,9 @@ void SearchEngine::updateNova() {
   // Set permissions
   QFile::Permissions perm=QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner | QFile::ReadUser | QFile::WriteUser | QFile::ExeUser | QFile::ReadGroup | QFile::ReadGroup;
   QFile(misc::qBittorrentPath()+"search_engine"+QDir::separator()+"nova2.py").setPermissions(perm);
-	filePath = misc::qBittorrentPath()+"search_engine"+QDir::separator()+"nova2dl.py";
-	if(misc::getPluginVersion(":/search_engine/nova2dl.py") > misc::getPluginVersion(filePath)) {
+
+  filePath = misc::qBittorrentPath()+"search_engine"+QDir::separator()+"nova2dl.py";
+  if(misc::getPluginVersion(":/search_engine/nova2dl.py") > misc::getPluginVersion(filePath)) {
     if(QFile::exists(filePath)){
       QFile::remove(filePath);
     }
@@ -323,7 +396,7 @@ void SearchEngine::updateNova() {
     }
     QFile::copy(":/search_engine/helpers.py", filePath);
   }
-	QFile(misc::qBittorrentPath()+"search_engine"+QDir::separator()+"helpers.py").setPermissions(perm);
+  QFile(misc::qBittorrentPath()+"search_engine"+QDir::separator()+"helpers.py").setPermissions(perm);
   QString destDir = misc::qBittorrentPath()+"search_engine"+QDir::separator()+"engines"+QDir::separator();
   QDir shipped_subDir(":/search_engine/engines/");
   QStringList files = shipped_subDir.entryList();
@@ -332,12 +405,12 @@ void SearchEngine::updateNova() {
     // Copy python classes
     if(file.endsWith(".py")) {
       if(misc::getPluginVersion(shipped_file) > misc::getPluginVersion(destDir+file) ) {
-        qDebug("shippped %s is more recent then local plugin, updating", file.toUtf8().data());
+        qDebug("shippped %s is more recent then local plugin, updating", file.toLocal8Bit().data());
         if(QFile::exists(destDir+file)) {
-          qDebug("Removing old %s", (destDir+file).toUtf8().data());
+          qDebug("Removing old %s", (destDir+file).toLocal8Bit().data());
           QFile::remove(destDir+file);
         }
-        qDebug("%s copied to %s", shipped_file.toUtf8().data(), (destDir+file).toUtf8().data());
+        qDebug("%s copied to %s", shipped_file.toLocal8Bit().data(), (destDir+file).toLocal8Bit().data());
         QFile::copy(shipped_file, destDir+file);
       }
     } else {
@@ -375,8 +448,10 @@ void SearchEngine::searchFinished(int exitcode,QProcess::ExitStatus){
   }
   if(currentSearchTab)
     currentSearchTab->getCurrentLabel()->setText(tr("Results", "i.e: Search results")+QString::fromUtf8(" <i>(")+misc::toQString(nb_search_results)+QString::fromUtf8(")</i>:"));
-  search_button->setEnabled(true);
-  stop_search_button->setEnabled(false);
+  search_button->setText("Search");
+  if(searchTimeout->isActive()) {
+    searchTimeout->stop();
+  }
 }
 
 // SLOT to append one line to search results list
@@ -387,62 +462,60 @@ void SearchEngine::appendSearchResult(QString line){
   if(parts.size() != 6){
     return;
   }
-  QString url = parts.takeFirst().trimmed();
-  QString filename = parts.first().trimmed();
-  parts << url;
+
   // Add item to search result list
-  int row = currentSearchTab->getCurrentSearchListModel()->rowCount();
-  currentSearchTab->getCurrentSearchListModel()->insertRow(row);
-  for(int i=0; i<6; ++i){
-    if(parts.at(i).trimmed().toFloat() == -1 && i != SIZE)
-      currentSearchTab->getCurrentSearchListModel()->setData(currentSearchTab->getCurrentSearchListModel()->index(row, i), tr("Unknown"));
-    else
-      currentSearchTab->getCurrentSearchListModel()->setData(currentSearchTab->getCurrentSearchListModel()->index(row, i), QVariant(parts.at(i).trimmed()));
+  QStandardItemModel *cur_model = currentSearchTab->getCurrentSearchListModel();
+  int row = cur_model->rowCount();
+  cur_model->insertRow(row);
+
+  cur_model->setData(cur_model->index(row, 5), parts.at(0).trimmed()); // download URL
+  cur_model->setData(cur_model->index(row, 0), parts.at(1).trimmed()); // Name
+  cur_model->setData(cur_model->index(row, 1), parts.at(2).trimmed().toLongLong()); // Size
+  bool ok = false;
+  qlonglong nb_seeders = parts.at(3).trimmed().toLongLong(&ok);
+  if(!ok || nb_seeders < 0) {
+    cur_model->setData(cur_model->index(row, 2), tr("Unknown")); // Seeders
+  } else {
+    cur_model->setData(cur_model->index(row, 2), nb_seeders); // Seeders
   }
-  // Add url to searchResultsUrls associative array
+  qlonglong nb_leechers = parts.at(4).trimmed().toLongLong(&ok);
+  if(!ok || nb_leechers < 0) {
+    cur_model->setData(cur_model->index(row, 3), tr("Unknown")); // Leechers
+  } else {
+    cur_model->setData(cur_model->index(row, 3), nb_leechers); // Leechers
+  }
+  cur_model->setData(cur_model->index(row, 4), parts.at(5).trimmed()); // Engine URL
+
   no_search_results = false;
   ++nb_search_results;
   // Enable clear & download buttons
   download_button->setEnabled(true);
 }
 
-// Stop search while it is working in background
-void SearchEngine::on_stop_search_button_clicked(){
-  // Kill process
-  searchProcess->terminate();
-  search_stopped = true;
-  searchTimeout->stop();
-}
-
 // Clear search results list
 void SearchEngine::closeTab_button_clicked(){
   if(all_tab.size()) {
     qDebug("currentTab rank: %d", tabWidget->currentIndex());
     qDebug("currentSearchTab rank: %d", tabWidget->indexOf(currentSearchTab));
     if(tabWidget->currentIndex() == tabWidget->indexOf(currentSearchTab)) {
-        qDebug("Deleted current search Tab");
-        if(searchProcess->state() != QProcess::NotRunning){
-            searchProcess->terminate();
-        }
-        if(searchTimeout->isActive()) {
-            searchTimeout->stop();
-        }
-        search_stopped = true;
-        currentSearchTab = 0;
+      qDebug("Deleted current search Tab");
+      if(searchProcess->state() != QProcess::NotRunning){
+        searchProcess->terminate();
+      }
+      if(searchTimeout->isActive()) {
+        searchTimeout->stop();
+      }
+      search_stopped = true;
+      currentSearchTab = 0;
     }
     delete all_tab.takeAt(tabWidget->currentIndex());
     if(!all_tab.size()) {
-        closeTab_button->setEnabled(false);
-        download_button->setEnabled(false);
+      closeTab_button->setEnabled(false);
+      download_button->setEnabled(false);
     }
   }
 }
 
-void SearchEngine::on_clearPatternButton_clicked() {
-  search_pattern->clear();
-  search_pattern->setFocus();
-}
-
 // Download selected items in search results list
 void SearchEngine::on_download_button_clicked(){
   //QModelIndexList selectedIndexes = currentSearchTab->getCurrentTreeView()->selectionModel()->selectedIndexes();
@@ -452,7 +525,7 @@ void SearchEngine::on_download_button_clicked(){
       // Get Item url
       QStandardItemModel *model = all_tab.at(tabWidget->currentIndex())->getCurrentSearchListModel();
       QString torrent_url = model->data(model->index(index.row(), URL_COLUMN)).toString();
-			QString engine_url = model->data(model->index(index.row(), ENGINE_URL_COLUMN)).toString();
+      QString engine_url = model->data(model->index(index.row(), ENGINE_URL_COLUMN)).toString();
       downloadTorrent(engine_url, torrent_url);
       all_tab.at(tabWidget->currentIndex())->setRowColor(index.row(), "red");
     }
